@@ -30,9 +30,8 @@ if os.path.exists(xsl_dir):
   sys.exit(0)
 
 for i in candidate_xsldirs:
-  globs = glob.glob(i)
   # Crude method of preferring the highest version, when multiple exist
-  globs.sort()
+  globs = sorted(glob.glob(i))
   globs.reverse()
   for j in globs:
     if os.path.exists(os.path.join(j, 'html', 'docbook.xsl')):
