@@ -157,3 +157,15 @@ svn_fs_base__err_no_such_node_origin(svn_fs_t *fs, const char *node_id)
      _("No record in 'node-origins' table for node id '%s' in "
        "filesystem '%s'"), node_id, fs->path);
 }
+
+svn_error_t *
+svn_fs_base__err_no_such_checksum_rep(svn_fs_t *fs, svn_checksum_t *checksum)
+{
+  return
+    svn_error_createf
+    (SVN_ERR_FS_NO_SUCH_CHECKSUM_REP, 0,
+     _("No record in 'checksum-reps' table for checksum '%s' in "
+       "filesystem '%s'"), svn_checksum_to_cstring_display(checksum,
+                                                           fs->pool),
+                           fs->path);
+}
