@@ -54,13 +54,18 @@ static const char * const statements[] = {
   };
 
 
-svn_error_t *
-svn_fs_fs__open_rep_cache(svn_fs_t *fs,
-                          apr_pool_t *pool)
+static svn_error_t *
+open_rep_cache(void *baton,
+               apr_pool_t *pool)
 {
+  svn_fs_t *fs = baton;
   fs_fs_data_t *ffd = fs->fsap_data;
   const char *db_path;
 
+  /* Be idempotent. */
+  if (ffd->rep_cache_db)
+    return SVN_NO_ERROR;
+
   /* Open (or create) the sqlite database.  It will be automatically
      closed when fs->pool is destoyed. */
   db_path = svn_path_join(fs->path, REP_CACHE_DB_NAME, pool);
@@ -72,6 +77,15 @@ svn_fs_fs__open_rep_cache(svn_fs_t *fs,
   return SVN_NO_ERROR;
 }
 
+svn_error_t *
+svn_fs_fs__open_rep_cache(svn_fs_t *fs,
+                          apr_pool_t *pool)
+{
+  fs_fs_data_t *ffd = fs->fsap_data;
+  return svn_atomic__init_once(&ffd->rep_cache_db_opened,
+                               open_rep_cache, fs, pool);
+}
+
 svn_error_t *
 svn_fs_fs__get_rep_reference(representation_t **rep,
                              svn_fs_t *fs,
@@ -82,11 +96,9 @@ svn_fs_fs__get_rep_reference(representation_t **rep,
   svn_sqlite__stmt_t *stmt;
   svn_boolean_t have_row;
 
-  if (ffd->rep_cache_db == NULL)
-    {
-      *rep = NULL;
-      return SVN_NO_ERROR;
-    }
+  SVN_ERR_ASSERT(ffd->rep_sharing_allowed);
+  if (! ffd->rep_cache_db)
+    SVN_ERR(svn_fs_fs__open_rep_cache(fs, pool));
 
   /* We only allow SHA1 checksums in this table. */
   if (checksum->kind != svn_checksum_sha1)
@@ -124,8 +136,9 @@ svn_fs_fs__set_rep_reference(svn_fs_t *fs,
   representation_t *old_rep;
   svn_sqlite__stmt_t *stmt;
 
-  if (ffd->rep_cache_db == NULL)
-    return SVN_NO_ERROR;
+  SVN_ERR_ASSERT(ffd->rep_sharing_allowed);
+  if (! ffd->rep_cache_db)
+    SVN_ERR(svn_fs_fs__open_rep_cache(fs, pool));
 
   /* We only allow SHA1 checksums in this table. */
   if (rep->sha1_checksum == NULL)
