@@ -484,7 +484,7 @@ static volatile svn_atomic_t sqlite_init_state;
    manner. */
 /* Don't call this function directly!  Use svn_atomic__init_once(). */
 static svn_error_t *
-init_sqlite(apr_pool_t *pool)
+init_sqlite(void *baton, apr_pool_t *pool)
 {
   if (sqlite3_libversion_number() < SQLITE_VERSION_NUMBER) {
     return svn_error_createf(SVN_ERR_SQLITE_ERROR, NULL,
@@ -526,7 +526,8 @@ svn_sqlite__get_schema_version(int *version,
 {
   svn_sqlite__db_t db;
 
-  SVN_ERR(svn_atomic__init_once(&sqlite_init_state, init_sqlite, scratch_pool));
+  SVN_ERR(svn_atomic__init_once(&sqlite_init_state,
+                                init_sqlite, NULL, scratch_pool));
   SQLITE_ERR(sqlite3_open(path, &db.db3), &db);
   SVN_ERR(get_schema(version, &db, scratch_pool));
   SQLITE_ERR(sqlite3_close(db.db3), &db);
@@ -574,7 +575,8 @@ svn_sqlite__open(svn_sqlite__db_t **db, const char *path,
                  int latest_schema, const char * const *upgrade_sql,
                  apr_pool_t *result_pool, apr_pool_t *scratch_pool)
 {
-  SVN_ERR(svn_atomic__init_once(&sqlite_init_state, init_sqlite, scratch_pool));
+  SVN_ERR(svn_atomic__init_once(&sqlite_init_state,
+                                init_sqlite, NULL, scratch_pool));
 
   *db = apr_palloc(result_pool, sizeof(**db));
 
