@@ -2,7 +2,7 @@
  * liveprops.c: mod_dav_svn live property provider functions for Subversion
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2008 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -16,52 +16,53 @@
  * ====================================================================
  */
 
-
+#include <apr_tables.h>
 
 #include <httpd.h>
+#include <http_core.h>
 #include <util_xml.h>
-#include <apr_tables.h>
-#include <apr_md5.h>
 #include <mod_dav.h>
 
-#include "dav_svn.h"
-
+#include "svn_checksum.h"
 #include "svn_pools.h"
 #include "svn_time.h"
 #include "svn_dav.h"
-#include "svn_md5.h"
 #include "svn_props.h"
 
+#include "private/svn_dav_protocol.h"
+
+#include "dav_svn.h"
+
 
 /*
 ** The namespace URIs that we use. This list and the enumeration must
 ** stay in sync.
 */
-static const char * const dav_svn_namespace_uris[] =
+static const char * const namespace_uris[] =
 {
-    "DAV:",
-    SVN_DAV_PROP_NS_DAV,
+  "DAV:",
+  SVN_DAV_PROP_NS_DAV,
 
-    NULL        /* sentinel */
+  NULL        /* sentinel */
 };
 enum {
-    DAV_SVN_NAMESPACE_URI_DAV,  /* the DAV: namespace URI */
-    DAV_SVN_NAMESPACE_URI       /* the dav<->ra_dav namespace URI */
+  NAMESPACE_URI_DAV,  /* the DAV: namespace URI */
+  NAMESPACE_URI       /* the dav<->ra_dav namespace URI */
 };
 
 #define SVN_RO_DAV_PROP(name) \
-        { DAV_SVN_NAMESPACE_URI_DAV, #name, DAV_PROPID_##name, 0 }
+        { NAMESPACE_URI_DAV, #name, DAV_PROPID_##name, 0 }
 #define SVN_RW_DAV_PROP(name) \
-        { DAV_SVN_NAMESPACE_URI_DAV, #name, DAV_PROPID_##name, 1 }
+        { NAMESPACE_URI_DAV, #name, DAV_PROPID_##name, 1 }
 #define SVN_RO_DAV_PROP2(sym,name) \
-        { DAV_SVN_NAMESPACE_URI_DAV, #name, DAV_PROPID_##sym, 0 }
+        { NAMESPACE_URI_DAV, #name, DAV_PROPID_##sym, 0 }
 #define SVN_RW_DAV_PROP2(sym,name) \
-        { DAV_SVN_NAMESPACE_URI_DAV, #name, DAV_PROPID_##sym, 1 }
+        { NAMESPACE_URI_DAV, #name, DAV_PROPID_##sym, 1 }
 
 #define SVN_RO_SVN_PROP(sym,name) \
-        { DAV_SVN_NAMESPACE_URI, #name, SVN_PROPID_##sym, 0 }
+        { NAMESPACE_URI, #name, SVN_PROPID_##sym, 0 }
 #define SVN_RW_SVN_PROP(sym,name) \
-        { DAV_SVN_NAMESPACE_URI, #name, SVN_PROPID_##sym, 1 }
+        { NAMESPACE_URI, #name, SVN_PROPID_##sym, 1 }
 
 
 enum {
@@ -71,7 +72,8 @@ enum {
   SVN_PROPID_deadprop_count
 };
 
-static const dav_liveprop_spec dav_svn_props[] =
+
+static const dav_liveprop_spec props[] =
 {
   /* ### don't worry about these for a bit */
 #if 0
@@ -102,12 +104,6 @@ static const dav_liveprop_spec dav_svn_props[] =
   { 0 } /* sentinel */
 };
 
-static const dav_liveprop_group dav_svn_liveprop_group =
-{
-    dav_svn_props,
-    dav_svn_namespace_uris,
-    &dav_svn_hooks_liveprop
-};
 
 /* Set *PROPVAL to the value for the revision property PROPNAME on
    COMMITTED_REV, in the repository identified by RESOURCE, if
@@ -134,29 +130,16 @@ static const dav_liveprop_group dav_svn_liveprop_group =
    might leak protected information because we will only test the
    readability of a single changed path.
 */
-static svn_error_t *dav_svn_get_path_revprop(svn_string_t **propval,
-                                             const dav_resource *resource,
-                                             svn_revnum_t committed_rev,
-                                             const char *propname,
-                                             apr_pool_t *pool)
+static svn_error_t *
+get_path_revprop(svn_string_t **propval,
+                 const dav_resource *resource,
+                 svn_revnum_t committed_rev,
+                 const char *propname,
+                 apr_pool_t *pool)
 {
-  dav_svn_authz_read_baton arb;
-  svn_boolean_t allowed;
-  svn_fs_root_t *root;
-
   *propval = NULL;
 
-  arb.r = resource->info->r;
-  arb.repos = resource->info->repos;
-  SVN_ERR(svn_fs_revision_root(&root,
-                               resource->info->repos->fs,
-                               committed_rev, pool));
-  SVN_ERR(dav_svn_authz_read(&allowed,
-                             root,
-                             resource->info->repos_path,
-                             &arb, pool));
-
-  if (! allowed)
+  if (! dav_svn__allow_read(resource, committed_rev, pool))
     return SVN_NO_ERROR;
 
   /* Get the property of the created revision. The authz is already
@@ -168,9 +151,122 @@ static svn_error_t *dav_svn_get_path_revprop(svn_string_t **propval,
                                     NULL, NULL, pool);
 }
 
-static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
-                                           int propid, dav_prop_insert what,
-                                           apr_text_header *phdr)
+
+enum time_format {
+  time_format_iso8601,
+  time_format_rfc1123
+};
+
+
+/* Given a mod_dav_svn @a resource, set @a *timeval and @a *datestring
+   to the last-modified-time of the resource.  The datestring will be
+   formatted according to @a format.  Use @a pool for both
+   scratchwork, and to allocate @a *datestring.
+
+   If @a timeval or @a datestring is NULL, don't touch it.
+
+   Return zero on success, non-zero if an error occurs. */
+static int
+get_last_modified_time(const char **datestring,
+                       apr_time_t *timeval,
+                       const dav_resource *resource,
+                       enum time_format format,
+                       apr_pool_t *pool)
+{
+  svn_revnum_t committed_rev = SVN_INVALID_REVNUM;
+  svn_string_t *committed_date = NULL;
+  svn_error_t *serr;
+  apr_time_t timeval_tmp;
+
+  if ((datestring == NULL) && (timeval == NULL))
+    return 0;
+
+  if (resource->baselined && resource->type == DAV_RESOURCE_TYPE_VERSION)
+    {
+      /* A baseline URI. */
+      committed_rev = resource->info->root.rev;
+    }
+  else if (resource->type == DAV_RESOURCE_TYPE_REGULAR
+           || resource->type == DAV_RESOURCE_TYPE_WORKING
+           || resource->type == DAV_RESOURCE_TYPE_VERSION)
+    {
+      serr = svn_fs_node_created_rev(&committed_rev,
+                                     resource->info->root.root,
+                                     resource->info->repos_path, pool);
+      if (serr != NULL)
+        {
+          svn_error_clear(serr);
+          return 1;
+        }
+    }
+  else
+    {
+      /* unsupported resource kind -- has no mod-time */
+      return 1;
+    }
+
+  serr = get_path_revprop(&committed_date,
+                          resource,
+                          committed_rev,
+                          SVN_PROP_REVISION_DATE,
+                          pool);
+  if (serr)
+    {
+      svn_error_clear(serr);
+      return 1;
+    }
+
+  if (committed_date == NULL)
+    return 1;
+
+  /* return the ISO8601 date as an apr_time_t */
+  serr = svn_time_from_cstring(&timeval_tmp, committed_date->data, pool);
+  if (serr != NULL)
+    {
+      svn_error_clear(serr);
+      return 1;
+    }
+
+  if (timeval)
+    memcpy(timeval, &timeval_tmp, sizeof(*timeval));
+
+  if (! datestring)
+    return 0;
+
+  if (format == time_format_iso8601)
+    {
+      *datestring = committed_date->data;
+    }
+  else if (format == time_format_rfc1123)
+    {
+      apr_time_exp_t tms;
+      apr_status_t status;
+
+      /* convert the apr_time_t into an apr_time_exp_t */
+      status = apr_time_exp_gmt(&tms, timeval_tmp);
+      if (status != APR_SUCCESS)
+        return 1;
+
+      /* stolen from dav/fs/repos.c   :-)  */
+      *datestring = apr_psprintf(pool, "%s, %.2d %s %d %.2d:%.2d:%.2d GMT",
+                                 apr_day_snames[tms.tm_wday],
+                                 tms.tm_mday, apr_month_snames[tms.tm_mon],
+                                 tms.tm_year + 1900,
+                                 tms.tm_hour, tms.tm_min, tms.tm_sec);
+    }
+  else /* unknown time format */
+    {
+      return 1;
+    }
+
+  return 0;
+}
+
+static dav_prop_insert
+insert_prop(const dav_resource *resource,
+            int propid,
+            dav_prop_insert what,
+            apr_text_header *phdr)
 {
   const char *value = NULL;
   const char *s;
@@ -215,7 +311,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
            as 'last modified date'?  */
         const char *datestring;
         apr_time_t timeval;
-        enum dav_svn_time_format format;
+        enum time_format format;
 
         /* ### for now, our global VCC has no such property. */
         if (resource->type == DAV_RESOURCE_TYPE_PRIVATE
@@ -223,20 +319,20 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
           {
             return DAV_PROP_INSERT_NOTSUPP;
           }
-       
+
         if (propid == DAV_PROPID_creationdate)
           {
             /* Return an ISO8601 date; this is what the svn client
                expects, and rfc2518 demands it. */
-            format = dav_svn_time_format_iso8601;
+            format = time_format_iso8601;
           }
         else /* propid == DAV_PROPID_getlastmodified */
           {
-            format = dav_svn_time_format_rfc1123;
+            format = time_format_rfc1123;
           }
 
-        if (0 != dav_svn_get_last_modified_time(&datestring, &timeval,
-                                                resource, format, p))
+        if (0 != get_last_modified_time(&datestring, &timeval,
+                                        resource, format, p))
           {
             return DAV_PROP_INSERT_NOTDEF;
           }
@@ -246,7 +342,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
       }
 
     case DAV_PROPID_creator_displayname:
-      {        
+      {
         svn_revnum_t committed_rev = SVN_INVALID_REVNUM;
         svn_string_t *last_author = NULL;
 
@@ -278,17 +374,17 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
                 value = "###error###";
                 break;
               }
-          }        
+          }
         else
           {
             return DAV_PROP_INSERT_NOTSUPP;
           }
 
-        serr = dav_svn_get_path_revprop(&last_author,
-                                        resource,
-                                        committed_rev,
-                                        SVN_PROP_REVISION_AUTHOR,
-                                        p);
+        serr = get_path_revprop(&last_author,
+                                resource,
+                                committed_rev,
+                                SVN_PROP_REVISION_AUTHOR,
+                                p);
         if (serr)
           {
             /* ### what to do? */
@@ -312,7 +408,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
     case DAV_PROPID_getcontentlength:
       {
         svn_filesize_t len = 0;
-        
+
         /* our property, but not defined on collection resources */
         if (resource->collection || resource->baselined)
           return DAV_PROP_INSERT_NOTSUPP;
@@ -366,11 +462,11 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
 
             if (pval)
               mime_type = pval->data;
-            else if ((! resource->info->repos->is_svn_client) 
+            else if ((! resource->info->repos->is_svn_client)
                      && resource->info->r->content_type)
               mime_type = resource->info->r->content_type;
             else
-              mime_type = "text/plain"; /* default for file */
+              mime_type = "text/plain";
 
             if ((serr = svn_mime_type_validate(mime_type, p)))
               {
@@ -394,7 +490,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
           return DAV_PROP_INSERT_NOTSUPP;
         }
 
-      value = dav_svn_getetag(resource, p);
+      value = dav_svn__getetag(resource, p);
       break;
 
     case DAV_PROPID_auto_version:
@@ -412,9 +508,9 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
       /* ### whoops. also defined for a VCC. deal with it later. */
       if (resource->type != DAV_RESOURCE_TYPE_VERSION || !resource->baselined)
         return DAV_PROP_INSERT_NOTSUPP;
-      value = dav_svn_build_uri(resource->info->repos, DAV_SVN_BUILD_URI_BC,
-                                resource->info->root.rev, NULL,
-                                1 /* add_href */, p);
+      value = dav_svn__build_uri(resource->info->repos, DAV_SVN__BUILD_URI_BC,
+                                 resource->info->root.rev, NULL,
+                                 1 /* add_href */, p);
       break;
 
     case DAV_PROPID_checked_in:
@@ -433,10 +529,10 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
               value = "###error###";
               break;
             }
-          s = dav_svn_build_uri(resource->info->repos,
-                                DAV_SVN_BUILD_URI_BASELINE, 
-                                revnum, NULL, 0 /* add_href */, p);
-          value = apr_psprintf(p, "<D:href>%s</D:href>", 
+          s = dav_svn__build_uri(resource->info->repos,
+                                 DAV_SVN__BUILD_URI_BASELINE,
+                                 revnum, NULL, 0 /* add_href */, p);
+          value = apr_psprintf(p, "<D:href>%s</D:href>",
                                apr_xml_quote_string(p, s, 1));
         }
       else if (resource->type != DAV_RESOURCE_TYPE_REGULAR)
@@ -447,14 +543,14 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
       else
         {
           svn_revnum_t rev_to_use =
-            dav_svn_get_safe_cr(resource->info->root.root,
-                                resource->info->repos_path, p);
+            dav_svn__get_safe_cr(resource->info->root.root,
+                                 resource->info->repos_path, p);
 
-          s = dav_svn_build_uri(resource->info->repos,
-                                DAV_SVN_BUILD_URI_VERSION,
-                                rev_to_use, resource->info->repos_path,
+          s = dav_svn__build_uri(resource->info->repos,
+                                 DAV_SVN__BUILD_URI_VERSION,
+                                 rev_to_use, resource->info->repos_path,
                                 0 /* add_href */, p);
-          value = apr_psprintf(p, "<D:href>%s</D:href>", 
+          value = apr_psprintf(p, "<D:href>%s</D:href>",
                                apr_xml_quote_string(p, s, 1));
         }
       break;
@@ -465,9 +561,9 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
       /* ### note that a VCC (a special VCR) is defined as _PRIVATE for now */
       if (resource->type != DAV_RESOURCE_TYPE_REGULAR)
         return DAV_PROP_INSERT_NOTSUPP;
-      value = dav_svn_build_uri(resource->info->repos, DAV_SVN_BUILD_URI_VCC,
-                                SVN_IGNORED_REVNUM, NULL, 
-                                1 /* add_href */, p);
+      value = dav_svn__build_uri(resource->info->repos, DAV_SVN__BUILD_URI_VCC,
+                                 SVN_IGNORED_REVNUM, NULL,
+                                 1 /* add_href */, p);
       break;
 
     case DAV_PROPID_version_name:
@@ -492,7 +588,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
       else
         {
           svn_revnum_t committed_rev = SVN_INVALID_REVNUM;
-          
+
           /* Get the CR field out of the node's skel.  Notice that the
              root object might be an ID root -or- a revision root. */
           serr = svn_fs_node_created_rev(&committed_rev,
@@ -505,7 +601,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
               value = "###error###";
               break;
             }
-          
+
           /* Convert the revision into a quoted string */
           s = apr_psprintf(p, "%ld", committed_rev);
           value = apr_xml_quote_string(p, s, 1);
@@ -531,11 +627,11 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
               || resource->type == DAV_RESOURCE_TYPE_WORKING
               || resource->type == DAV_RESOURCE_TYPE_VERSION))
         {
-          unsigned char digest[APR_MD5_DIGESTSIZE];
+          svn_checksum_t *checksum;
 
-          serr = svn_fs_file_md5_checksum(digest,
-                                          resource->info->root.root,
-                                          resource->info->repos_path, p);
+          serr = svn_fs_file_checksum(&checksum, svn_checksum_md5,
+                                      resource->info->root.root,
+                                      resource->info->repos_path, TRUE, p);
           if (serr != NULL)
             {
               /* ### what to do? */
@@ -544,7 +640,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
               break;
             }
 
-          value = svn_md5_digest_to_cstring(digest, p);
+          value = svn_checksum_to_cstring(checksum, p);
 
           if (! value)
             return DAV_PROP_INSERT_NOTSUPP;
@@ -568,8 +664,8 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
     case SVN_PROPID_deadprop_count:
       {
         unsigned int propcount;
-        apr_hash_t *proplist;  
-      
+        apr_hash_t *proplist;
+
         if (resource->type != DAV_RESOURCE_TYPE_REGULAR)
           return DAV_PROP_INSERT_NOTSUPP;
 
@@ -581,14 +677,14 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
             /* ### what to do? */
             svn_error_clear(serr);
             value = "###error###";
-            break;  
+            break;
           }
-         
+
         propcount = apr_hash_count(proplist);
         value = apr_psprintf(p, "%u", propcount);
         break;
       }
-      
+
     default:
       /* ### what the heck was this property? */
       return DAV_PROP_INSERT_NOTDEF;
@@ -597,7 +693,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
   /* assert: value != NULL */
 
   /* get the information and global NS index for the property */
-  global_ns = dav_get_liveprop_info(propid, &dav_svn_liveprop_group, &info);
+  global_ns = dav_get_liveprop_info(propid, &dav_svn__liveprop_group, &info);
 
   /* assert: info != NULL && info->name != NULL */
 
@@ -615,7 +711,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
     s = apr_psprintf(response_pool,
                      "<D:supported-live-property D:name=\"%s\" "
                      "D:namespace=\"%s\"/>" DEBUG_CR,
-                     info->name, dav_svn_namespace_uris[info->ns]);
+                     info->name, namespace_uris[info->ns]);
   }
   apr_text_append(response_pool, phdr, s);
 
@@ -623,62 +719,86 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
   return what;
 }
 
-static int dav_svn_is_writable(const dav_resource *resource, int propid)
+
+static int
+is_writable(const dav_resource *resource, int propid)
 {
   const dav_liveprop_spec *info;
 
-  (void) dav_get_liveprop_info(propid, &dav_svn_liveprop_group, &info);
+  (void) dav_get_liveprop_info(propid, &dav_svn__liveprop_group, &info);
   return info->is_writable;
 }
 
-static dav_error * dav_svn_patch_validate(const dav_resource *resource,
-                                          const apr_xml_elem *elem,
-                                          int operation, void **context,
-                                          int *defer_to_dead)
+
+static dav_error *
+patch_validate(const dav_resource *resource,
+               const apr_xml_elem *elem,
+               int operation,
+               void **context,
+               int *defer_to_dead)
 {
   /* NOTE: this function will not be called unless/until we have
      modifiable (writable) live properties. */
   return NULL;
 }
 
-static dav_error * dav_svn_patch_exec(const dav_resource *resource,
-                                      const apr_xml_elem *elem,
-                                      int operation, void *context,
-                                      dav_liveprop_rollback **rollback_ctx)
+
+static dav_error *
+patch_exec(const dav_resource *resource,
+           const apr_xml_elem *elem,
+           int operation,
+           void *context,
+           dav_liveprop_rollback **rollback_ctx)
 {
   /* NOTE: this function will not be called unless/until we have
      modifiable (writable) live properties. */
   return NULL;
 }
 
-static void dav_svn_patch_commit(const dav_resource *resource,
-                                 int operation, void *context,
-                                 dav_liveprop_rollback *rollback_ctx)
+
+static void
+patch_commit(const dav_resource *resource,
+             int operation,
+             void *context,
+             dav_liveprop_rollback *rollback_ctx)
 {
   /* NOTE: this function will not be called unless/until we have
      modifiable (writable) live properties. */
 }
 
-static dav_error * dav_svn_patch_rollback(const dav_resource *resource,
-                                          int operation, void *context,
-                                          dav_liveprop_rollback *rollback_ctx)
+static dav_error *
+patch_rollback(const dav_resource *resource,
+                       int operation,
+                       void *context,
+                       dav_liveprop_rollback *rollback_ctx)
 {
   /* NOTE: this function will not be called unless/until we have
      modifiable (writable) live properties. */
   return NULL;
 }
 
-const dav_hooks_liveprop dav_svn_hooks_liveprop = {
-  dav_svn_insert_prop,
-  dav_svn_is_writable,
-  dav_svn_namespace_uris,
-  dav_svn_patch_validate,
-  dav_svn_patch_exec,
-  dav_svn_patch_commit,
-  dav_svn_patch_rollback,
+
+static const dav_hooks_liveprop hooks_liveprop = {
+  insert_prop,
+  is_writable,
+  namespace_uris,
+  patch_validate,
+  patch_exec,
+  patch_commit,
+  patch_rollback,
 };
 
-void dav_svn_gather_propsets(apr_array_header_t *uris)
+
+const dav_liveprop_group dav_svn__liveprop_group =
+{
+  props,
+  namespace_uris,
+  &hooks_liveprop
+};
+
+
+void
+dav_svn__gather_propsets(apr_array_header_t *uris)
 {
   /* ### what should we use for a URL to describe the available prop set? */
   /* ### for now... nothing. we will *only* have DAV properties */
@@ -688,152 +808,58 @@ void dav_svn_gather_propsets(apr_array_header_t *uris)
 #endif
 }
 
-int dav_svn_find_liveprop(const dav_resource *resource,
-                          const char *ns_uri, const char *name,
-                          const dav_hooks_liveprop **hooks)
+
+int
+dav_svn__find_liveprop(const dav_resource *resource,
+                       const char *ns_uri,
+                       const char *name,
+                       const dav_hooks_liveprop **hooks)
 {
   /* don't try to find any liveprops if this isn't "our" resource */
-  if (resource->hooks != &dav_svn_hooks_repos)
+  if (resource->hooks != &dav_svn__hooks_repository)
     return 0;
 
-  return dav_do_find_liveprop(ns_uri, name, &dav_svn_liveprop_group, hooks);
-}
-
-void dav_svn_insert_all_liveprops(request_rec *r, const dav_resource *resource,
-                                  dav_prop_insert what, apr_text_header *phdr)
-{
-    const dav_liveprop_spec *spec;
-    apr_pool_t *pool;
-    apr_pool_t *subpool;
-
-    /* don't insert any liveprops if this isn't "our" resource */
-    if (resource->hooks != &dav_svn_hooks_repos)
-        return;
-
-    if (!resource->exists) {
-        /* a lock-null resource */
-        /*
-        ** ### technically, we should insert empty properties. dunno offhand
-        ** ### what part of the spec said this, but it was essentially thus:
-        ** ### "the properties should be defined, but may have no value".
-        */
-        return;
-    }
-
-    pool = resource->info->pool;
-    subpool = svn_pool_create(pool);
-    resource->info->pool = subpool;
-
-    for (spec = dav_svn_props; spec->name != NULL; ++spec)
-      {
-        svn_pool_clear(subpool);
-        (void) dav_svn_insert_prop(resource, spec->propid, what, phdr);
-      }
-
-    resource->info->pool = pool;
-    svn_pool_destroy(subpool);
-
-    /* ### we know the others aren't defined as liveprops */
-}
-
-void dav_svn_register_uris(apr_pool_t *p)
-{
-    /* register the namespace URIs */
-    dav_register_liveprop_group(p, &dav_svn_liveprop_group);
+  return dav_do_find_liveprop(ns_uri, name, &dav_svn__liveprop_group, hooks);
 }
 
 
-int dav_svn_get_last_modified_time(const char **datestring,
-                                   apr_time_t *timeval,
-                                   const dav_resource *resource,
-                                   enum dav_svn_time_format format,
-                                   apr_pool_t *pool)
+void
+dav_svn__insert_all_liveprops(request_rec *r,
+                              const dav_resource *resource,
+                              dav_prop_insert what,
+                              apr_text_header *phdr)
 {
-  svn_revnum_t committed_rev = SVN_INVALID_REVNUM;
-  svn_string_t *committed_date = NULL;
-  svn_error_t *serr;
-  apr_time_t timeval_tmp;
+  const dav_liveprop_spec *spec;
+  apr_pool_t *pool;
+  apr_pool_t *subpool;
 
-  if ((datestring == NULL) && (timeval == NULL))
-    return 0;
+  /* don't insert any liveprops if this isn't "our" resource */
+  if (resource->hooks != &dav_svn__hooks_repository)
+    return;
 
-  if (resource->baselined && resource->type == DAV_RESOURCE_TYPE_VERSION)
+  if (!resource->exists)
     {
-      /* A baseline URI. */
-      committed_rev = resource->info->root.rev;
-    }
-  else if (resource->type == DAV_RESOURCE_TYPE_REGULAR
-           || resource->type == DAV_RESOURCE_TYPE_WORKING
-           || resource->type == DAV_RESOURCE_TYPE_VERSION)
-    {
-      serr = svn_fs_node_created_rev(&committed_rev,
-                                     resource->info->root.root,
-                                     resource->info->repos_path, pool);
-      if (serr != NULL)
-        {
-          svn_error_clear(serr);
-          return 1;
-        }
-    }
-  else
-    {
-      /* unsupported resource kind -- has no mod-time */
-      return 1;
+      /* a lock-null resource */
+      /*
+       ** ### technically, we should insert empty properties. dunno offhand
+       ** ### what part of the spec said this, but it was essentially thus:
+       ** ### "the properties should be defined, but may have no value".
+       */
+      return;
     }
 
-  serr = dav_svn_get_path_revprop(&committed_date,
-                                  resource,
-                                  committed_rev,
-                                  SVN_PROP_REVISION_DATE,
-                                  pool);
-  if (serr)
+  pool = resource->info->pool;
+  subpool = svn_pool_create(pool);
+  resource->info->pool = subpool;
+
+  for (spec = props; spec->name != NULL; ++spec)
     {
-      svn_error_clear(serr);
-      return 1;
+      svn_pool_clear(subpool);
+      (void) insert_prop(resource, spec->propid, what, phdr);
     }
 
-  if (committed_date == NULL)
-    return 1;
+  resource->info->pool = pool;
+  svn_pool_destroy(subpool);
 
-  /* return the ISO8601 date as an apr_time_t */
-  serr = svn_time_from_cstring(&timeval_tmp, committed_date->data, pool);
-  if (serr != NULL)
-    {
-      svn_error_clear(serr);
-      return 1;
-    }
-
-  if (timeval)
-    memcpy(timeval, &timeval_tmp, sizeof(*timeval));
-
-  if (! datestring)
-    return 0;
-
-  if (format == dav_svn_time_format_iso8601)
-    {
-      *datestring = committed_date->data;
-    }
-  else if (format == dav_svn_time_format_rfc1123)
-    {
-      apr_time_exp_t tms;
-      apr_status_t status;
-      
-      /* convert the apr_time_t into an apr_time_exp_t */
-      status = apr_time_exp_gmt(&tms, timeval_tmp);
-      if (status != APR_SUCCESS)
-        return 1;
-              
-      /* stolen from dav/fs/repos.c   :-)  */
-      *datestring = apr_psprintf(pool, "%s, %.2d %s %d %.2d:%.2d:%.2d GMT",
-                                 apr_day_snames[tms.tm_wday],
-                                 tms.tm_mday, apr_month_snames[tms.tm_mon],
-                                 tms.tm_year + 1900,
-                                 tms.tm_hour, tms.tm_min, tms.tm_sec);      
-    }
-  else /* unknown time format */
-    {
-      return 1;
-    }
-
-  return 0;
-}                                              
+  /* ### we know the others aren't defined as liveprops */
+}
