@@ -49,7 +49,7 @@ def build_repos(sbox):
 
 def run_sync(url, expected_error=None):
   "Synchronize the mirror repository with the master"
-  output, errput = svntest.main.run_svnsync(
+  exit_code, output, errput = svntest.main.run_svnsync(
     "synchronize", url,
     "--username", svntest.main.wc_author,
     "--password", svntest.main.wc_passwd)
@@ -69,7 +69,7 @@ def run_sync(url, expected_error=None):
 
 def run_copy_revprops(url, expected_error=None):
   "Copy revprops to the mirror repository from the master"
-  output, errput = svntest.main.run_svnsync(
+  exit_code, output, errput = svntest.main.run_svnsync(
     "copy-revprops", url,
     "--username", svntest.main.wc_author,
     "--password", svntest.main.wc_passwd)
@@ -90,7 +90,7 @@ def run_copy_revprops(url, expected_error=None):
 
 def run_init(dst_url, src_url):
   "Initialize the mirror repository from the master"
-  output, errput = svntest.main.run_svnsync(
+  exit_code, output, errput = svntest.main.run_svnsync(
     "initialize", dst_url, src_url,
     "--username", svntest.main.wc_author,
     "--password", svntest.main.wc_passwd)
@@ -99,6 +99,28 @@ def run_init(dst_url, src_url):
   if output != ['Copied properties for revision 0.\n']:
     raise SVNUnexpectedStdout(output)
 
+def run_info(url, expected_error=None):
+  "Print synchronization information of the repository"
+  exit_code, output, errput = svntest.main.run_svnsync(
+    "info", url,
+    "--username", svntest.main.wc_author,
+    "--password", svntest.main.wc_passwd)
+  if errput:
+    if expected_error is None:
+      raise SVNUnexpectedStderr(errput)
+    else:
+      expected_error = svntest.verify.RegexOutput(expected_error,
+                                                  match_all=False)
+      svntest.verify.compare_and_display_lines(None, "STDERR",
+                                               expected_error, errput)
+  elif expected_error is not None:
+    raise SVNExpectedStderr
+  if not output and not expected_error:
+    # should be: ['From URL: http://....\n',
+    #             'From UUID: XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX\n',
+    #             'Last Merged Revision: XXX\n']
+    raise SVNUnexpectedStdout("Missing stdout")
+
 
 def run_test(sbox, dump_file_name, subdir = None, exp_dump_file_name = None):
   """Load a dump file, sync repositories, and compare contents with the original
@@ -112,7 +134,8 @@ or another dump file."""
                                    'svnsync_tests_data')
   # Load the specified dump file into the master repository.
   master_dumpfile_contents = file(os.path.join(svnsync_tests_dir,
-                                               dump_file_name)).readlines()
+                                               dump_file_name),
+                                  'rb').readlines()
   svntest.actions.run_and_verify_load(sbox.repo_dir, master_dumpfile_contents)
 
   # Create the empty destination repository.
@@ -120,11 +143,10 @@ or another dump file."""
   build_repos(dest_sbox)
 
   # Setup the mirror repository.  Feed it the UUID of the source repository.
-  output, errput = svntest.main.run_svnlook("uuid", sbox.repo_dir)
-  mirror_cfg = ["SVN-fs-dump-format-version: 2\n",
-                "UUID: " + output[0],
-                ]
-  svntest.actions.run_and_verify_load(dest_sbox.repo_dir, mirror_cfg)
+  exit_code, output, errput = svntest.main.run_svnlook("uuid", sbox.repo_dir)
+  svntest.actions.run_and_verify_svnadmin2("Setting UUID", None, None, 0,
+                                           'setuuid', dest_sbox.repo_dir,
+                                           output[0][:-1])
 
   # Create the revprop-change hook for this test
   svntest.actions.enable_revprop_changes(dest_sbox.repo_dir)
@@ -148,7 +170,7 @@ or another dump file."""
   # Create a dump file from the mirror repository.
   dest_dump = svntest.actions.run_and_verify_dump(dest_sbox.repo_dir)
 
-  # Compare the dump produced by the mirror repository with either the original 
+  # Compare the dump produced by the mirror repository with either the original
   # dump file (used to create the master repository) or another specified dump
   # file.
   if exp_dump_file_name:
@@ -408,11 +430,11 @@ def copy_from_unreadable_dir(sbox):
     '\n', # log message is stripped
   ]
 
-  out, err = svntest.main.run_svn(None,
-                                  'log',
-                                  '-r', '3',
-                                  '-v',
-                                  dest_sbox.repo_url)
+  exit_code, out, err = svntest.main.run_svn(None,
+                                             'log',
+                                             '-r', '3',
+                                             '-v',
+                                             dest_sbox.repo_url)
 
   if err:
     raise SVNUnexpectedStderr(err)
@@ -533,11 +555,11 @@ def copy_with_mod_from_unreadable_dir(sbox):
     '\n', # log message is stripped
   ]
 
-  out, err = svntest.main.run_svn(None,
-                                  'log',
-                                  '-r', '2',
-                                  '-v',
-                                  dest_sbox.repo_url)
+  exit_code, out, err = svntest.main.run_svn(None,
+                                             'log',
+                                             '-r', '2',
+                                             '-v',
+                                             dest_sbox.repo_url)
 
   if err:
     raise SVNUnexpectedStderr(err)
@@ -635,11 +657,11 @@ def copy_with_mod_from_unreadable_dir_and_copy(sbox):
     '\n', # log message is stripped
   ]
 
-  out, err = svntest.main.run_svn(None,
-                                  'log',
-                                  '-r', '2',
-                                  '-v',
-                                  dest_sbox.repo_url)
+  exit_code, out, err = svntest.main.run_svn(None,
+                                             'log',
+                                             '-r', '2',
+                                             '-v',
+                                             dest_sbox.repo_url)
 
   if err:
     raise SVNUnexpectedStderr(err)
@@ -667,12 +689,12 @@ def copy_revprops(sbox):
 
 def only_trunk(sbox):
   "test syncing subdirectories"
-  run_test(sbox, "svnsync-trunk-only.dump", "/trunk", 
+  run_test(sbox, "svnsync-trunk-only.dump", "/trunk",
            "svnsync-trunk-only.expected.dump")
 
 def only_trunk_A_with_changes(sbox):
   "test syncing subdirectories with changes on root"
-  run_test(sbox, "svnsync-trunk-A-changes.dump", "/trunk/A", 
+  run_test(sbox, "svnsync-trunk-A-changes.dump", "/trunk/A",
            "svnsync-trunk-A-changes.expected.dump")
 
 # test for issue #2904
@@ -680,6 +702,66 @@ def move_and_modify_in_the_same_revision(sbox):
   "test move parent and modify child file in same rev"
   run_test(sbox, "svnsync-move-and-modify.dump")
 
+def info_synchronized(sbox):
+  "test info cmd on a synchronized repo"
+
+  sbox.build("svnsync-info-syncd", False)
+
+  # Get the UUID of the source repository.
+  exit_code, output, errput = svntest.main.run_svnlook("uuid", sbox.repo_dir)
+  src_uuid = output[0].strip()
+
+  dest_sbox = sbox.clone_dependent()
+  build_repos(dest_sbox)
+
+  svntest.actions.enable_revprop_changes(dest_sbox.repo_dir)
+  run_init(dest_sbox.repo_url, sbox.repo_url)
+  run_sync(dest_sbox.repo_url)
+
+  exit_code, output, errput = svntest.main.run_svnsync(
+    "info", dest_sbox.repo_url,
+    "--username", svntest.main.wc_author,
+    "--password", svntest.main.wc_passwd)
+  if errput:
+      raise SVNUnexpectedStderr(errput)
+
+  expected_out = ['Source URL: %s\n' % sbox.repo_url,
+                  'Source Repository UUID: %s\n' % src_uuid,
+                  'Last Merged Revision: 1\n',
+                  ]
+
+  svntest.verify.compare_and_display_lines(None,
+                                           'INFO',
+                                           expected_out,
+                                           output)
+
+def info_not_synchronized(sbox):
+  "test info cmd on an un-synchronized repo"
+
+  sbox.build("svnsync-info-not-syncd", False)
+
+  run_info(sbox.repo_url,
+           ".*Repository '%s' is not initialized.*" % sbox.repo_url)
+
+#----------------------------------------------------------------------
+
+def copy_bad_line_endings(sbox):
+  "copy with inconsistent lineendings in svn:props"
+  run_test(sbox, "copy-bad-line-endings.dump",
+           exp_dump_file_name="copy-bad-line-endings.expected.dump")
+
+#----------------------------------------------------------------------
+
+def delete_svn_props(sbox):
+  "copy with svn:prop deletions"
+  run_test(sbox, "delete-svn-props.dump")
+
+def commit_a_copy_of_root(sbox):
+  "commit a copy of root causes sync to fail"
+  #Testcase for issue 3438.
+  run_test(sbox, "repo_with_copy_of_root_dir.dump")
+
+
 ########################################################################
 # Run the tests
 
@@ -713,6 +795,11 @@ test_list = [ None,
               SkipUnless(only_trunk_A_with_changes,
                          server_has_partial_replay),
               move_and_modify_in_the_same_revision,
+              info_synchronized,
+              info_not_synchronized,
+              copy_bad_line_endings,
+              delete_svn_props,
+              commit_a_copy_of_root,
              ]
 
 if __name__ == '__main__':
