@@ -1,7 +1,7 @@
 /* fs-test.c --- tests for the filesystem
  *
  * ====================================================================
- * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2008 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -19,13 +19,13 @@
 #include <string.h>
 #include <apr_pools.h>
 
+#include "../svn_test.h"
+
 #include "svn_pools.h"
 #include "svn_time.h"
 #include "svn_string.h"
 #include "svn_fs.h"
-#include "svn_md5.h"
 
-#include "../svn_test.h"
 #include "../svn_test_fs.h"
 
 #include "../../libsvn_fs_base/id.h"
@@ -58,8 +58,8 @@ create_berkeley_filesystem(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create and close a repository. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-create-berkeley",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-create-berkeley", opts,
+                                  pool));
 
   return SVN_NO_ERROR;
 }
@@ -88,8 +88,8 @@ open_berkeley_filesystem(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create and close a repository (using fs). */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-open-berkeley",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-open-berkeley", opts,
+                                  pool));
 
   /* Create a different fs object, and use it to re-open the
      repository again.  */
@@ -206,7 +206,7 @@ check_id(svn_fs_t *fs, const svn_fs_id_t *id, svn_boolean_t *present,
 
   args.id = id;
   args.fs = fs;
-  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_check_id, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_check_id, &args, TRUE, pool));
 
   if (args.present)
     *present = TRUE;
@@ -278,8 +278,8 @@ abort_txn(const char **msg,
     return SVN_NO_ERROR;
 
   /* Prepare two txns to receive the Greek tree. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-abort-txn",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-abort-txn", opts,
+                                  pool));
   SVN_ERR(svn_fs_begin_txn(&txn1, fs, 0, pool));
   SVN_ERR(svn_fs_begin_txn(&txn2, fs, 0, pool));
   SVN_ERR(svn_fs_txn_root(&txn1_root, txn1, pool));
@@ -517,8 +517,8 @@ delete_mutables(const char **msg,
     return SVN_NO_ERROR;
 
   /* Prepare a txn to receive the greek tree. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-del-from-dir",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-del-from-dir", opts,
+                                  pool));
   SVN_ERR(svn_fs_begin_txn(&txn, fs, 0, pool));
   SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
 
@@ -713,8 +713,8 @@ delete(const char **msg,
    */
 
   /* Prepare a txn to receive the greek tree. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-del-tree",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-del-tree", opts,
+                                  pool));
   SVN_ERR(svn_fs_begin_txn(&txn, fs, 0, pool));
   SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
 
@@ -1200,8 +1200,8 @@ create_within_copy(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create a filesystem and repository. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-create-within-copy",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-create-within-copy", opts,
+                                  pool));
 
   /*** Revision 1:  Create the greek tree in revision.  ***/
   SVN_ERR(svn_fs_begin_txn(&txn, fs, youngest_rev, spool));
@@ -1329,8 +1329,8 @@ skip_deltas(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create a filesystem and repository. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-skip-deltas",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-skip-deltas", opts,
+                                  pool));
 
   /* Create the file. */
   SVN_ERR(svn_fs_begin_txn(&txn, fs, youngest_rev, subpool));
@@ -1408,8 +1408,8 @@ redundant_copy(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create a filesystem and repository. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-redundant-copy",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-redundant-copy", opts,
+                                  pool));
 
   /* Create the greek tree in revision 1. */
   SVN_ERR(svn_fs_begin_txn(&txn, fs, youngest_rev, pool));
@@ -1429,7 +1429,7 @@ redundant_copy(const char **msg,
   args.fs = fs;
   args.txn_name = txn_name;
   args.txn = &transaction;
-  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_get_txn, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_get_txn, &args, FALSE, pool));
   if (transaction->copies->nelts != 1)
     return svn_error_createf(SVN_ERR_TEST_FAILED, NULL,
                              "Expected 1 copy; got %d",
@@ -1443,7 +1443,7 @@ redundant_copy(const char **msg,
 
   /* Now, examine the transaction.  There should still only have been
      one copy operation that "took". */
-  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_get_txn, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_get_txn, &args, FALSE, pool));
   if (transaction->copies->nelts != 1)
     return svn_error_createf(SVN_ERR_TEST_FAILED, NULL,
                              "Expected only 1 copy; got %d",
@@ -1463,6 +1463,72 @@ redundant_copy(const char **msg,
 }
 
 
+static svn_error_t *
+orphaned_textmod_change(const char **msg,
+                        svn_boolean_t msg_only,
+                        svn_test_opts_t *opts,
+                        apr_pool_t *pool)
+{
+  apr_pool_t *subpool = svn_pool_create(pool);
+  svn_fs_t *fs;
+  svn_fs_txn_t *txn;
+  svn_fs_root_t *txn_root, *root;
+  svn_revnum_t youngest_rev = 0;
+  svn_txdelta_window_handler_t wh_func;
+  void *wh_baton;
+  apr_hash_t *changed_paths;
+
+  *msg = "test for orphaned textmod changed paths";
+  if (msg_only)
+    return SVN_NO_ERROR;
+
+  /* Create a filesystem and repository. */
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-orphaned-changes", opts,
+                                  pool));
+
+  /* Revision 1:  Create and commit the greek tree. */
+  SVN_ERR(svn_fs_begin_txn(&txn, fs, 0, subpool));
+  SVN_ERR(svn_fs_txn_root(&txn_root, txn, subpool));
+  SVN_ERR(svn_test__create_greek_tree(txn_root, subpool));
+  SVN_ERR(svn_fs_commit_txn(NULL, &youngest_rev, txn, subpool));
+  svn_pool_clear(subpool);
+
+  /* Revision 2:  Start to change "iota", but don't complete the work. */
+  SVN_ERR(svn_fs_begin_txn(&txn, fs, youngest_rev, subpool));
+  SVN_ERR(svn_fs_txn_root(&txn_root, txn, subpool));
+  SVN_ERR(svn_fs_apply_textdelta
+          (&wh_func, &wh_baton, txn_root, "iota", NULL, NULL, subpool));
+
+  /* Don't send any delta windows, but do commit the transaction.
+     According to the FS API docs, this is not a legal codepath.  But
+     this requirement on the API was added *after* its BDB
+     implementation, and the BDB backend can't enforce compliance with
+     the additional API rules in this case.  So we are really just
+     testing that misbehaving callers don't introduce more damage to
+     the repository than they have to. */
+  SVN_ERR(svn_fs_commit_txn(NULL, &youngest_rev, txn, subpool));
+  svn_pool_clear(subpool);
+
+  /* Fetch changed paths for the youngest revision.  We should find none. */
+  SVN_ERR(svn_fs_revision_root(&root, fs, youngest_rev, subpool));
+  SVN_ERR(svn_fs_paths_changed(&changed_paths, root, subpool));
+  if (apr_hash_count(changed_paths) != 0)
+    {
+      svn_fs_path_change_t *change = apr_hash_get(changed_paths, "/iota",
+                                                  APR_HASH_KEY_STRING);
+      if (change && change->text_mod)
+        return svn_error_create(SVN_ERR_TEST_FAILED, NULL,
+                                "Got unexpected textmods changed path "
+                                "for 'iota'");
+      else
+        return svn_error_create(SVN_ERR_TEST_FAILED, NULL,
+                                "Got non-empty changed paths hash where empty "
+                                "one expected");
+    }
+
+  return SVN_NO_ERROR;
+}
+
 /* ------------------------------------------------------------------------ */
 
 /* The test table.  */
@@ -1479,5 +1545,6 @@ struct svn_test_descriptor_t test_funcs[] =
     SVN_TEST_PASS(canonicalize_abspath),
     SVN_TEST_PASS(skip_deltas),
     SVN_TEST_PASS(redundant_copy),
+    SVN_TEST_PASS(orphaned_textmod_change),
     SVN_TEST_NULL
   };
